require 'spec_helper'

class ExampleClass
  def object_method
    "object_method"
  end

  def set_object_instance_variable(val)
    @object_instance_var = val
  end

  def get_object_instance_variable
    @object_instance_var
  end
end

class ExampleRole < Rietveld::Role
  def role_method
    object_method
  end

  def set_role_instance_variable(val)
    @role_instance_var = val
  end

  def get_role_instance_variable
    @role_instance_var
  end
end

describe Rietveld::Role do
  let(:object) { ExampleClass.new }
  let(:role)   { ExampleRole.new(object) }

  context 'straight delegates' do
    example '#==' do
      expect(role == object).to be(object == object)
    end

    example '#equal?' do
      expect(role.equal?(object)).to be(object.equal?(object))
    end

    example '#eql?' do
      expect(role.eql?(object)).to be(object.eql?(object))
    end

    example '#class' do
      expect(role.class).to be(object.class)
      expect(role.class).to be(ExampleClass)
    end
  end

  context 'hybrid delegates' do
    describe '#methods' do
      it 'should include methods of the role_player' do
        expect(role.methods).to include(*object.methods)
      end

      it 'should include methods of the role' do
        expect(role.methods).to include(*role.role_methods)
      end

      it 'should be a union of role_player and role methods' do
        methods = role.role_methods | object.methods
        expect(role.methods).to contain_exactly(*methods)
      end
    end
  end

  context 'delegation' do
    it 'delegates to object methods' do
      expect(role.object_method).to eq(object.object_method)
    end

    it 'responds to object methods' do
      expect(role).to respond_to(:object_method)
    end
  end

  context 'ivars' do
    context 'belonging to object' do
      before { object.set_object_instance_variable('test') }

      it 'has #instance_variable_defined?' do
        expect(role).to be_instance_variable_defined('@object_instance_var')
      end

      it 'can access with #instance_variable_get' do
        expect(role.instance_variable_get('@object_instance_var')).to eq('test')
      end

      it 'can change with #instance_variable_set' do
        role.instance_variable_set('@object_instance_var', 'changed')
        expect(object.get_object_instance_variable).to eq('changed')
      end
    end

    context 'belonging to role' do
      before { role.set_role_instance_variable('test') }

      it 'has #instance_variable_defined?' do
        expect(role).to be_instance_variable_defined('@role_instance_var')
      end

      it 'object has not #instance_variable_defined?' do
        expect(object).to_not be_instance_variable_defined('@role_instance_var')
      end

      it 'can access with #instance_variable_get' do
        expect(role.instance_variable_get('@role_instance_var')).to eq('test')
      end

      it 'can change with #instance_variable_set' do
        role.instance_variable_set('@role_instance_var', 'changed')
        expect(role.get_role_instance_variable).to eq('changed')
      end
    end
  end
end
